@@ -221,41 +221,41 @@ per_language_title_sort_articles = {
         'epo': (r'La\s+', r"L'", 'L´'),
         # Spanish
         'spa': (r'El\s+', r'La\s+', r'Lo\s+', r'Los\s+', r'Las\s+', r'Un\s+',
-                  r'Una\s+', r'Unos\s+', r'Unas\s+'),
+                r'Una\s+', r'Unos\s+', r'Unas\s+'),
         # French
         'fra': (r'Le\s+', r'La\s+', r"L'", r'L´', r'L’', r'Les\s+', r'Un\s+', r'Une\s+',
-                  r'Des\s+', r'De\s+La\s+', r'De\s+', r"D'", r'D´', r'D’'),
+                r'Des\s+', r'De\s+La\s+', r'De\s+', r"D'", r'D´', r'D’'),
         # Polish
         'pol': (),
         # Italian
-        'ita': ('Lo\\s+', 'Il\\s+', "L'", 'L´', 'La\\s+', 'Gli\\s+',
-                'I\\s+', 'Le\\s+', 'Uno\\s+', 'Un\\s+', 'Una\\s+', "Un'",
-                'Un´', 'Dei\\s+', 'Degli\\s+', 'Delle\\s+', 'Del\\s+',
-                'Della\\s+', 'Dello\\s+', "Dell'", 'Dell´'),
+        'ita': (r'Lo\s+', r'Il\s+', r"L'", r'L´', r'La\s+', r'Gli\s+',
+                r'I\s+', r'Le\s+', r'Uno\s+', r'Un\s+', r'Una\s+', "rUn'",
+                r'Un´', r'Dei\s+', r'Degli\s+', r'Delle\s+', r'Del\s+',
+                r'Della\s+', r'Dello\s+', r"Dell'", r'Dell´'),
         # Portuguese
         'por': (r'A\s+', r'O\s+', r'Os\s+', r'As\s+', r'Um\s+', r'Uns\s+',
-                  r'Uma\s+', r'Umas\s+', ),
+                r'Uma\s+', r'Umas\s+'),
         # Romanian
-        'ron': (r'Un\s+', r'O\s+', r'Nişte\s+', ),
+        'ron': (r'Un\s+', r'O\s+', r'Nişte\s+'),
         # German
         'deu': (r'Der\s+', r'Die\s+', r'Das\s+', r'Den\s+', r'Ein\s+',
-                  r'Eine\s+', r'Einen\s+', r'Dem\s+', r'Des\s+', r'Einem\s+',
-                  r'Eines\s+'),
+                r'Eine\s+', r'Einen\s+', r'Dem\s+', r'Des\s+', r'Einem\s+',
+                r'Eines\s+'),
         # Dutch
         'nld': (r'De\s+', r'Het\s+', r'Een\s+', r"'n\s+", r"'s\s+", r'Ene\s+',
-                  r'Ener\s+', r'Enes\s+', r'Den\s+', r'Der\s+', r'Des\s+',
-                  r"'t\s+"),
+                r'Ener\s+', r'Enes\s+', r'Den\s+', r'Der\s+', r'Des\s+',
+                r"'t\s+"),
         # Swedish
-        'swe': (r'En\s+', r'Ett\s+', r'Det\s+', r'Den\s+', r'De\s+', ),
+        'swe': (r'En\s+', r'Ett\s+', r'Det\s+', r'Den\s+', r'De\s+'),
         # Turkish
-        'tur': (r'Bir\s+', ),
+        'tur': (r'Bir\s+',),
         # Afrikaans
-        'afr': (r"'n\s+", r'Die\s+', ),
+        'afr': (r"'n\s+", r'Die\s+'),
         # Greek
         'ell': (r'O\s+', r'I\s+', r'To\s+', r'Ta\s+', r'Tus\s+', r'Tis\s+',
-                  r"'Enas\s+", r"'Mia\s+", r"'Ena\s+", r"'Enan\s+", ),
+                r"'Enas\s+", r"'Mia\s+", r"'Ena\s+", r"'Enan\s+"),
         # Hungarian
-        'hun': (r'A\s+', r'Az\s+', r'Egy\s+',),
+        'hun': (r'A\s+', r'Az\s+', r'Egy\s+'),
 }
 default_language_for_title_sort = None
 title_sort_articles = r'^(A|The|An)\s+'
@@ -427,7 +427,7 @@ maximum_cover_size = (1650, 2200)
 # control where it is sent. Valid values are "main", "carda", "cardb". Note
 # that if there isn't enough free space available on the location you choose,
 # the files will be sent to the location with the most free space.
-send_news_to_device_location = "main"
+send_news_to_device_location = 'main'
 
 #: Unified toolbar on macOS
 # If you enable this option and restart calibre, the toolbar will be 'unified'
@@ -580,3 +580,15 @@ openers_by_scheme = {}
 # It must be one of the values Default, Sunday, Monday, Tuesday, Wednesday,
 # Thursday, Friday, or Saturday, all in English, spelled exactly as shown.
 calendar_start_day_of_week = 'Default'
+
+#: East Asian language to use for transliteration
+# Setting this tweak will make calibre use the specified language as the "base"
+# language when transliterating East Asian languages to English. This might be
+# useful if you run calibre in English but want text transliterated to
+# Japanese instead of Chinese. The valid values are 'ja' for Japanese, 'kr' for Korean,
+# 'vn' for Vietnamese, 'zh' for Chinese.
+# Any other value will use the language set in calibre preferences as the base
+# language. A base language other than those in the above list causes transliteration
+# with a base language of Chinese.
+# Example: east_asian_base_language = 'ja'
+east_asian_base_language = ''
